"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2023, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const azdev = require("azure-devops-node-api");
const utils_1 = require("./utils");
class AzureDevOps {
    constructor(authorizationToken, organizationName) {
        this.authorizationToken = authorizationToken;
        this.organizationName = organizationName;
        this.orgUrl = `https://dev.azure.com/${this.organizationName}`;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const authHandler = azdev.getPersonalAccessTokenHandler(this.authorizationToken);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call, @typescript-eslint/no-unsafe-member-access
        this.connection = new azdev.WebApi(this.orgUrl, authHandler);
    }
    async getPrUrl(prId, projectName) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const gitApi = await this.connection.getGitApi();
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const prProperties = await gitApi.getPullRequestById(prId, projectName);
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions, @typescript-eslint/no-unsafe-member-access
        return `${prProperties?.repository?.webUrl}/pullrequest/${prId}`;
    }
    async fetchPrComments(projectName, repoName, prId) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const existingThreads = await // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
         
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        (await this.connection.getGitApi()).getThreads(repoName, prId, projectName);
        const commentsByFileArray = [];
        existingThreads.forEach((thread) => {
            if (thread) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                const threadContext = thread.threadContext;
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                if (threadContext && !thread.isDeleted) {
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                    const primaryComment = thread.comments[0];
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
                    const indexOfExistingFile = commentsByFileArray.findIndex(
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
                    (element) => element.fileName === threadContext.filePath.substring(1));
                    if (indexOfExistingFile === -1) {
                        commentsByFileArray.push({
                            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
                            fileName: threadContext.filePath.substring(1),
                            comments: [
                                {
                                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                                    lineNumber: thread.threadContext.rightFileStart.line,
                                    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                                    content: primaryComment.content,
                                },
                            ],
                        });
                    }
                    else {
                        commentsByFileArray[indexOfExistingFile].comments.push({
                            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                            lineNumber: thread.threadContext.rightFileStart.line,
                            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                            content: primaryComment.content,
                        });
                    }
                }
            }
        });
        return commentsByFileArray;
    }
    async postPrComment(projectName, repoName, prId, commentsPerFiles
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    ) {
        const fetchResultPromises = [];
        if (commentsPerFiles) {
            commentsPerFiles.forEach((commentsPerFile) => {
                if (commentsPerFile.comments) {
                    // create comment threads per file
                    fetchResultPromises.concat(this.sendRequestPostPRComments(repoName, prId, projectName, commentsPerFile));
                }
            });
            if (fetchResultPromises) {
                const responses = await Promise.all(fetchResultPromises);
                const responseJsonPromises = [];
                responses.forEach((response) => {
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
                    responseJsonPromises.push(response.json());
                });
                // TO-DO - add logic to ensure post is successful
                // (await Promise.all(responseJsonPromises)).forEach((responseJson) => {
                //   console.log(responseJson);
                // });
            }
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    async sendRequestPostPRComments(repoName, prId, projectName, commentsPerFile) {
        const resultPromises = [];
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const gitAPI = await this.connection.getGitApi();
        commentsPerFile.comments.forEach((comment) => {
            const newCommentThread = {};
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            newCommentThread.comments = [
                {
                    parentCommentId: 0,
                    commentType: 1,
                    content: comment.content.replace(new RegExp('\\\\n', 'g'), '\n'),
                },
            ];
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            newCommentThread.status = 1;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            newCommentThread.threadContext = {
                filePath: `/${commentsPerFile.fileName}`,
                rightFileStart: {
                    line: comment.startLineNumber,
                    offset: 0,
                },
                rightFileEnd: {
                    // line: comment.endLineNumber,
                    line: comment.startLineNumber,
                },
            };
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
            resultPromises.push(gitAPI.createThread(newCommentThread, repoName, prId, projectName));
        });
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return resultPromises;
    }
    async getPrDiff(projectName, prId, gitFolderPath = './') {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const prInfo = await // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
         
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        (await this.connection.getGitApi()).getPullRequestById(prId, projectName);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-unsafe-member-access
        const destinationBranchName = `origin/${(0, utils_1.removeRefHeadFromBranchName)(prInfo.targetRefName)}`;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-unsafe-member-access
        const sourceBranchName = `origin/${(0, utils_1.removeRefHeadFromBranchName)(prInfo.sourceRefName)}`;
        const diffFilesLinesadded = (0, utils_1.parseDiff)(gitFolderPath, destinationBranchName, sourceBranchName);
        return diffFilesLinesadded;
    }
}
exports.default = AzureDevOps;
//# sourceMappingURL=azureDevOps.js.map