"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const node_fetch_1 = require("node-fetch");
const core_1 = require("@salesforce/core");
const utils_1 = require("../helper/utils");
// eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-var-requires
const parser = require('git-pr-diff-parser');
// TO-DO - Error handling
class BitbucketDevOps {
    // private projectName: string;
    // private repoName: string;
    constructor(username, password, workspaceName) {
        this.PAGELENGTH = 50;
        this.username = username;
        this.password = password;
        this.workspaceName = workspaceName;
        this.baseUrl = `${BitbucketDevOps.BITBUCKET_API_URL}/repositories/${this.workspaceName}`;
    }
    async fetchPrComments(repoSlugName, prId) {
        const existingCommentsByFiles = [];
        const options = {
            method: 'GET',
            headers: {
                Authorization: 'Basic ' + Buffer.from(this.username + ':' + this.password).toString('base64'),
                Accept: 'application/json',
            },
        };
        const url = `${this.baseUrl}/${repoSlugName}/pullrequests/${prId}/comments?pagelen=${this.PAGELENGTH}`;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call
        const response = await (0, node_fetch_1.default)(url, options);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const data = await response.json();
        let currentPage = 1;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        const totalSize = data.size;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        const pageLength = data.pagelen;
        const totalPages = Math.ceil(totalSize / pageLength);
        const fetchedCommentsPromisesArray = [];
        (0, utils_1.printMessage)(`Pulling ${totalSize} inline comments from PR ${prId}`, utils_1.MessageType.Info, core_1.Logger.getRoot().getLevel());
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
        let comments = data.values;
        while (currentPage < totalPages) {
            currentPage++;
            // eslint-disable-next-line @typescript-eslint/no-unsafe-call
            fetchedCommentsPromisesArray.push((0, node_fetch_1.default)(`${url}&page=${currentPage}`, options));
        }
        const fetchedCommentsArray = await Promise.all(fetchedCommentsPromisesArray);
        const dataPromises = [];
        fetchedCommentsArray.forEach((fetchedComments) => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
            dataPromises.push(fetchedComments.json());
        });
        const dataArray = await Promise.all(dataPromises);
        // eslint-disable-next-line @typescript-eslint/no-shadow
        dataArray.forEach((data) => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
            comments = [...comments, ...data.values];
        });
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        comments.forEach((comment) => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
            if (comment.inline) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                const indexOfExistingFile = existingCommentsByFiles.findIndex(
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                (element) => element.fileName === comment.inline.path);
                if (indexOfExistingFile > -1) {
                    const matchedElement = existingCommentsByFiles[indexOfExistingFile];
                    if (!matchedElement.comments) {
                        matchedElement.comments = [];
                    }
                    matchedElement.comments.push({
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                        startLineNumber: comment.inline.from,
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                        endLineNumber: comment.inline.to,
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                        lineNumber: comment.inline.from || comment.inline.to,
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                        htmlContent: comment.content.html,
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                        content: comment.content.raw,
                    });
                    existingCommentsByFiles[indexOfExistingFile] = matchedElement;
                }
                else {
                    existingCommentsByFiles.push({
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                        fileName: comment.inline.path,
                        comments: [
                            {
                                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                                startLineNumber: comment.inline.from,
                                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                                endLineNumber: comment.inline.to,
                                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                                lineNumber: comment.inline.from || comment.inline.to,
                                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                                htmlContent: comment.content.html,
                                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
                                content: comment.content.raw,
                            },
                        ],
                    });
                }
            }
        });
        if (existingCommentsByFiles.length > 0) {
            (0, utils_1.printMessage)(`Total files with comments - ${existingCommentsByFiles.length}`, utils_1.MessageType.Info, core_1.Logger.getRoot().getLevel());
        }
        else {
            (0, utils_1.printMessage)('No comments found for the PR', utils_1.MessageType.Warn, core_1.Logger.getRoot().getLevel());
        }
        return existingCommentsByFiles;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async postPrComment(repoSlugName, prId, commentsPerFiles) {
        const fetchResultPromises = [];
        if (commentsPerFiles) {
            commentsPerFiles.forEach((commentsPerFile) => {
                if (commentsPerFile.comments) {
                    commentsPerFile.comments.forEach((comment) => {
                        const requestBody = `{"content":{"raw":"${comment.content}"}, "inline":{"to":${comment.lineNumber}, "path":"${commentsPerFile.fileName}"}}`;
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                        fetchResultPromises.push(this.sendRequestPostPRComments(repoSlugName, prId, JSON.parse(requestBody)));
                    });
                }
            });
            if (fetchResultPromises) {
                const responses = await Promise.all(fetchResultPromises);
                const responseJsonPromises = [];
                responses.forEach((response) => {
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
                    responseJsonPromises.push(response.json());
                });
                // TO-DO - add logic to ensure post is successful
                // (await Promise.all(responseJsonPromises)).forEach((responseJson) => {
                //   console.log(responseJson);
                // });
            }
        }
    }
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type, @typescript-eslint/require-await
    async sendRequestPostPRComments(repoName, prId, body) {
        const options = {
            method: 'POST',
            headers: {
                Authorization: 'Basic ' + Buffer.from(this.username + ':' + this.password).toString('base64'),
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(body),
        };
        const url = `${this.baseUrl}/${repoName}/pullrequests/${prId}/comments`;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call
        const response = (0, node_fetch_1.default)(url, options);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return response;
    }
    async getPrDiff(repoName, prId, gitFolderPath = './') {
        const options = {
            method: 'GET',
            headers: {
                Authorization: 'Basic ' + Buffer.from(this.username + ':' + this.password).toString('base64'),
                'Content-Type': 'application/json',
            },
        };
        const url = `${this.baseUrl}/${repoName}/pullrequests/${prId}`;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        const { destination, source } = await (await (0, node_fetch_1.default)(url, options)).json();
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions, @typescript-eslint/no-unsafe-member-access
        const destinationBranchName = `origin/${destination.branch.name}`;
        // eslint-disable-next-line @typescript-eslint/restrict-template-expressions, @typescript-eslint/no-unsafe-member-access
        const sourceBranchName = `origin/${source.branch.name}`;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-call
        const parsedDiff = await parser(gitFolderPath, destinationBranchName, sourceBranchName);
        const diffFilesLinesadded = [];
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
        parsedDiff.forEach((eachFile) => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call
            let addedLine = eachFile.lines.filter((line) => {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-return, @typescript-eslint/no-unsafe-member-access
                if (line.type === 'add')
                    return line;
            });
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            if (addedLine && addedLine.length > 0) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-return, @typescript-eslint/no-unsafe-call
                addedLine = addedLine.map((line) => line.ln);
            }
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-member-access
            diffFilesLinesadded.push({ fileName: eachFile.to, linesAdded: addedLine });
        });
        return diffFilesLinesadded;
    }
}
exports.default = BitbucketDevOps;
BitbucketDevOps.BITBUCKET_API_URL = 'https://api.bitbucket.org/2.0';
BitbucketDevOps.BITBUCKET_WEBAPP_URL = 'https://bitbucket.org';
//# sourceMappingURL=bitbucketDevOps.js.map